<?php

namespace Myriad\Authentication\Lib\Guards;

use Illuminate\Auth\Events\Authenticated;
use Illuminate\Auth\TokenGuard;
use Illuminate\Contracts\Auth\Authenticatable;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Installations\InstallationManager;

class AccessTokenGuard extends TokenGuard
{
    const AUTH_HEADER = 'Authorization';
    const TOKEN_REGEX_PATTERN = '/[tT]oken ([^\s]+)/';
    const TOKEN_MATCH_INDEX = 1;

    /**
     * Get the currently authenticated user.
     *
     * @return Authenticatable|null
     */
    public function user(): ?Authenticatable
    {
        // Return the user if we have one already authenticated
        if ($this->user !== null) {
            return $this->user;
        }

        $user = null;

        $token = $this->getTokenForRequest();

        if (!empty($token)) {
            $accessToken = app(IAccessTokenRepository::class)->getAccessTokenByToken($token);

            if ($this->hasTokenWithValidUuidHeader($accessToken)) {
                $user = $this->provider->retrieveByCredentials(
                    [$this->storageKey => $token]
                );
                $this->fireAuthenticatedEvent($user);
            }
        }

        return $this->user = $user;
    }

    /**
     * Get the token for the current request.
     *
     * @return string
     */
    public function getTokenForRequest(): string
    {
        $matches = [];

        if ($this->isValidToken($matches)) {
            return $matches[self::TOKEN_MATCH_INDEX];
        }

        return '';
    }

    /**
     * @param  Authenticatable  $user
     * @return void
     */
    protected function fireAuthenticatedEvent($user)
    {
        event(new Authenticated('token', $user));
    }

    /**
     * @param  array  $matches
     * @return false|int
     */
    private function isValidToken(&$matches)
    {
        $authHeader = $this->request->header(self::AUTH_HEADER);
        return preg_match(self::TOKEN_REGEX_PATTERN, $authHeader, $matches);
    }

    /**
     * @param $accessToken
     * @return bool
     */
    private function hasTokenWithValidUuidHeader($accessToken): bool
    {
        $uuidFromRequest = app(InstallationManager::class)->getUuidFromRequest();
        return !empty($accessToken) && $accessToken->client->uuid == $uuidFromRequest;
    }
}
