<?php

namespace Myriad\Authentication\Lib\Strategies;

use Illuminate\Contracts\Config\Repository as Config;
use Illuminate\Contracts\Events\Dispatcher;
use Illuminate\Contracts\View\Factory as ViewFactory;
use Myriad\Authentication\Lib\Events\PhoneNumberVerified;
use Myriad\Authentication\Lib\Repositories\IUserRepository;
use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Lib\Services\ITextMessageSender;
use Myriad\Authentication\Models\UserPhone;

class PhoneNumberVerification implements IVerificationStrategy, IPhoneNumberConfirmer
{
    use CreatesVerificationCodes;
    use VerifiesUserContactMethod;

    private $textMessageSender;
    private $viewFactory;
    private $dispatcher;

    /**
     * @param  IUserRepository  $userRepository
     */
    public function __construct(
        Config $config,
        ITextMessageSender $textMessageSender,
        ViewFactory $viewFactory,
        IUserRepository $userRepository,
        Dispatcher $dispatcher
    ) {
        $this->textMessageSender = $textMessageSender;
        $this->viewFactory = $viewFactory;
        $this->dispatcher = $dispatcher;

        // set the properties necessary for VerifiesUserContactMethod
        $this->config = $config;
        $this->userRepository = $userRepository;
        $this->model = UserPhone::class;
        $this->property = "phone";
        $this->contactCollection = "phoneNumbers";
        $this->contactAlreadyExistsError = "Phone number belongs to a different user!";
    }

    public function sendConfirmationTextMessage($phoneNumber, $user, array $attributes)
    {
        $attributes = [
            'type' => array_key_exists('type', $attributes) ? $attributes['type'] : 'personal',
            'primary' => array_key_exists('primary', $attributes) ? $attributes['primary'] : false,
            'user_id' => is_null($user) ? null : $user->getAuthIdentifier(),
        ];

        // create a verification code
        $code = $this->createVerificationCode($phoneNumber, PhoneNumberVerification::class, $attributes);

        // get the text message template
        $verificationTemplate = $this->config->get('authentication.text_messages.templates.phone_verification.template');

        // app_name for template
        $appName = $this->config->get('authentication.text_messages.app_name');

        // create a view from the template
        $view = $this->viewFactory->make($verificationTemplate, ['code' => $code->code, 'app_name' => $appName]);

        // render the view
        $message = $view->render();

        // get the from phone number
        $fromNumber = $this->config->get('authentication.text_messages.from_number');

        $this->textMessageSender->sendTextMessage($fromNumber, $phoneNumber, $message);
    }

    public function sendAutofillConfirmationTextMessage($phoneNumber, $user, array $attributes)
    {
        $attributes = [
            'type' => array_key_exists('type', $attributes) ? $attributes['type'] : 'personal',
            'primary' => array_key_exists('primary', $attributes) ? $attributes['primary'] : false,
            'user_id' => is_null($user) ? null : $user->getAuthIdentifier(),
        ];

        // create a verification code
        $code = $this->createAutofillVerificationCode($phoneNumber, PhoneNumberVerification::class, $attributes);

        // get the text message template
        $verificationTemplate = $this->config->get('authentication.text_messages.templates.phone_verification.template');

        // app_name for template
        $appName = $this->config->get('authentication.text_messages.app_name');

        // create a view from the template
        $view = $this->viewFactory->make($verificationTemplate, ['code' => $code->code, 'app_name' => $appName]);

        // render the view
        $message = $view->render();

        // get the from phone number
        $fromNumber = $this->config->get('authentication.text_messages.from_number');

        $this->textMessageSender->sendTextMessage($fromNumber, $phoneNumber, $message);
    }

    /**
     * Raise a phone number verified event
     *
     * @param  UserPhone  $userPhone
     */
    public function raiseContactVerifiedEvent($userPhone)
    {
        $this->dispatcher->dispatch(new PhoneNumberVerified($userPhone));
    }
}
