<?php


namespace Myriad\Authentication\Models;

use Illuminate\Database\Eloquent\Relations\HasMany;

trait LookupByEmailOrPhone
{

    /**
     * Get all the user's phone numbers
     *
     * @return HasMany|UserPhone[]
     */
    public function phoneNumbers(): HasMany
    {
        return $this->hasMany(UserPhone::class);
    }

    /**
     * Get all the user's email addresses
     *
     * @return HasMany|UserEmail[]
     */
    public function emailAddresses(): HasMany
    {
        return $this->hasMany(UserEmail::class);
    }

    public function lookup(array $data)
    {
        $user = $this->tryRetrieveByEmail($data);

        if (is_null($user)) {
            $user = $this->tryRetrieveByPhone($data);
        }

        return $user;
    }

    protected function tryRetrieveByEmail(array $data)
    {
        if (!$this->containsEmail($data)) {
            return null;
        }

        return $this->newQuery()->whereHas('emailAddresses', function ($query) use ($data) {
            return $query->where('email', $data['email'])
                ->where('primary', true);
        })->first();
    }

    protected function tryRetrieveByPhone(array $data)
    {
        if (!$this->containsPhone($data)) {
            return null;
        }

        return $this->newQuery()->whereHas('phoneNumbers', function ($query) use ($data) {
            return $query->where('phone', $data['phone'])
                ->where('primary', true);
        })->first();
    }


    /**
     * Is there a non-null email in the data array
     *
     * @param  array  $data
     * @return bool
     */
    protected function containsEmail(array $data): bool
    {
        return $this->dataContainsKey($data, 'email');
    }

    /**
     * Is there a non-null phone in the data array
     *
     * @param  array  $data
     * @return bool
     */
    protected function containsPhone(array $data): bool
    {
        return $this->dataContainsKey($data, 'phone');
    }

    /**
     * Does the data contain a non-null value for key
     *
     * @param  array  $data
     * @param  mixed  $key
     * @return bool
     */
    protected function dataContainsKey(array $data, $key): bool
    {
        if (empty($data)) {
            return false;
        }

        if (!array_key_exists($key, $data) || is_null($data[$key])) {
            return false;
        }

        return true;
    }
}
